module ssmodule
	use genLRmodule
	use globals
	implicit none
	
	contains

	function ssglr result(out)
		use rnfdf_int
		use rnstt_int
		
		integer, parameter	:: nsim=15000,n=250
		real				:: Ys(n,nsim), lengths(nsim), u(n),y2(n)
		logical				:: covers(nsim)
		real				:: z(k),zs,zk,CI(2),q
		integer				:: nmiss
		integer				:: nsample=n
		real				:: h
        real, parameter		:: qlist(2,2)=[8.05261, 2.27221, 12.1895, 3.23361]
		
		real, allocatable	:: out(:,:)

		integer				:: l,i,ih

		h=htarget
		out=reshape([real::],[2,0])
		do i=1,6
			do l=1,nsim
				select case(i)
					case(1)
						call rnnoa(Ys(:,l))
				   		q=anorin(1-h/nsample)		
				        if(.not.quantileflag) q=(nsample/h)/(exp(q**2/2.)*Sqrt(2*Pi))
					case(2)
						call rnstt(3.0,Ys(:,l))
						q=tin(1-h/nsample,3.0)
						if(.not.quantileflag) q=(nsample/h)*(3*Sqrt(3.0))/(3*Pi + Pi*q**2)
					case(3)
						call rnnoa(Ys(:,l))
						Ys(:,l)=exp(Ys(:,l))
				   		q=exp(anorin(1-h/nsample))
				        if(.not.quantileflag) q=(nsample/h)*(exp(0.5)*erfc((-1 + Log(q))/Sqrt(2.0)))/2.
					case(4)
						call rnfdf(4.0,4.0,Ys(:,l))
						q=fin(1-h/nsample,4.0,4.0)
						if(.not.quantileflag) q=(nsample/h)*(2*(1 + 3*q + 3*q**2))/(1 + q)**3
					case(5)
						call rnun(u)
						call rnstt(3.0,Ys(:,l))
						call rnnoa(y2)
						Ys(:,l)=merge(y2,Ys(:,l),u<0.80)
				        ih=count(h+.001>[0.1,5.0])
						q=qlist(ih,1+count([.not.quantileflag]))
					case(6)
						call rnun(u)
						Ys(:,l)=merge(-1+sqrt(2*U),1-sqrt((1-U)*2),u<.5)
						q=1 - Sqrt(2 - 2*(1 - h/nsample))
						if(.not.quantileflag) q=(nsample/h)*(1 - 3*q**2 + 2*q**3)/6.
				end select
			enddo
!$omp parallel do private(z,zs,zk,CI)
			do l=1,nsim
				call erset(0,0,0)
				call ordst(-Ys(:,l),k,z(1:k),iopt=1,nmiss=nmiss)
				z(1:k)=-z(1:k)
				zs=z(1)-z(k)
				zk=z(k)
				z=(z-zk)/zs
			
				CI=zk+zs*getgenLR_CI(Z)
				covers(l)=CI(1)<q .and. q<CI(2)
				lengths(l)=CI(2)-CI(1)
			enddo
			out=out.clr.[real(count(covers))/nsim,sum(lengths)/nsim]
			call mdisp(out)
		enddo
	end function

end module

program mfort
	use globals
	use compute
	use genLRmodule
	use ssmodule
	implicit none
	
	integer				:: i
	integer, parameter	:: nh=17,nk=9,nalpha=4
	real, parameter		:: hlist(nh)=[(exp(-5.0+0.5*(i-1)),i=1,nh)]
	
	real, parameter		:: alphalist(nalpha)=[0.01,.05,0.10,0.20]
	integer, parameter	:: klist(nk)=[5,10,15,20,30,40,50,75,100]
	
	integer, parameter	:: t1klist(3)=[10,20,50]
	real, parameter		:: t1hlist(2)=[0.1,5.0]
	
	integer				:: ih,ik,ia,iq
	real, allocatable	:: outtab0(:,:), outtab1(:,:), profcvs(:,:), outtabgLR(:,:),outtabgLRc(:,:)
	character(len=150)	:: ffolder="C:/Dropbox/mystuff/tail/JASARev/out/"
	
	call disp_set(advance="double",orient="row")
	call inittime
    call rnopt(6)
	call rnset(13)
    call openML

	call prep
! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOO  Small SampleOOOOOOOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
	call setdens
	call setwalt
	call setlam
	call adjustlam
	call chksize
	call setgenLR
	call mdisp(ssglr())
	
! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOO  Table 1 OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO

	alpha=0.05
	allocate(outtab1(0,10))
	allocate(outtabgLR(0,12))
	
	do iq=1,2
		quantileflag=(iq==1)
	do ik=1,3
		k=t1klist(ik)
	outtab0=reshape([real::],[3,0])
	outtabgLRc=reshape([real::],[2,0])
	do ih=1,2
		htarget=t1hlist(ih)
	
		
		call rnset(14)
		call prep
		print *,"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
		print *,"entering ", quantileflag,htarget, k
		print *,"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
		call setdens
		call setwalt
		call setlam
		call adjustlam
		call chksize
		call setgenLR
		outtab0=outtab0.clr.selxsitab()
		outtabgLRc=outtabgLRc.clr.ssglr()
	enddo
	outtab1=outtab1.cud.outtab0
	outtabglr=outtabglr.cud.outtabglrc
	call mdisp(outtab1)
	call mdisp(outtabglrc)
	enddo
	enddo

! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOO  all combinations OOOOOOOOOOOOOOOOOOOOOO
! OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
	
	
	print *,"hlist"
	call disp(hlist)
	

	outtab0=reshape([-1.0,-1.0,-1.0,-1.0,appgrid],[1,napp+4])
	outtab1=reshape([-1.0,-1.0,-1.0,-1.0,altgrid],[1,nalt+4])
	outtabgLR=reshape([real::],[0,6])
	
	do iq=1,2
		quantileflag=(iq==1)
	do ih=1,nh
		htarget=hlist(ih)
	do ik=1,nk
		k=klist(ik)
		if(htarget-1E-5>k) cycle

	    call rnset(14)
		call prep
		print *,"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
		print *,"entering ", quantileflag,htarget, k
		print *,"XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"
		call setdens

		do ia=1,nalpha
			alpha=alphalist(ia)
			
!			call printtime
			call setwalt
			call setlam
			call adjustlam
			call chksize
			call setgenLR
!			print *,getss()
			outtab0=outtab0.cud.[real(iq-1),htarget,real(k),alpha,lam]
			outtab1=outtab1.cud.[real(iq-1),htarget,real(k),alpha,walt]
			outtabgLR=outtabgLR.cud.[real(iq-1),htarget,real(k),alpha,cvgenLR,rlgenLR]

!			call disp(outtab0)
!			call disp(outtab1)
!			print *,"now saving"
			call savematML(trim(ffolder)//"nulltab",outtab0)
			call savematML(trim(ffolder)//"alttab",outtab1)
			call savematML(trim(ffolder)//"genLRtab",outtabgLR)
		enddo
	enddo
	call printtime
	enddo
	enddo
end program

function ext_f(s) result(val)
    use compute
    implicit none
    real    :: s,val
    val=exp(intfunc(s,globalz,globalxsi)-globalls)
end function
    
function ext_minuslogf(s) result(val)
    use compute
	USE, INTRINSIC :: IEEE_ARITHMETIC
    implicit none
    real    :: s,val
    val=-intfunc(s,globalz,globalxsi)
    if(printflag) call disp([s,-val])
	if(IEEE_IS_FINITE(val)) return
!$omp critical
	print *,"warning: logdens is not finite"	
	print *,"draw number", gl
	print *,"xsi",globalxsi, "val",val, "s",s, "q", getqxsi(globalxsi)
	call disp([globalz])
	call savevec("c:/dropbox/mystuff/tail/text.txt",globalz)
	stop
!$omp end critical
end function

function ext_minusdlogf(s) result(val)
    use compute
	USE, INTRINSIC :: IEEE_ARITHMETIC
    implicit none
    real    :: s,val
    val=-dintfunc(s,globalz,globalxsi)
    if(printflag) then
        print *,"deriv"
        call disp([s,-val])
	endif
	if(IEEE_IS_FINITE(val)) return
!$omp critical
	print *,globalxsi
	print *,"error: derivative is not finite"	
	print *,"xsi",globalxsi, "val",val, "s",s, "q", getqxsi(globalxsi)
	call disp([globalz])
	call savevec("c:/dropbox/mystuff/tail/text.txt",globalz)
	print *,"h",htarget,"k",k
	stop
!$omp end critical
end function

	