% function used to compute confidence interval for extreme quantile or tail
% conditional expectation with generalized LR method
% input: X: data (1 by k row vector, descendingly ordered) 
%        h: interested in 1-h/n qunatile or TCE
%        alpha: level of significance
%        est_tce: indicator 0 for quantile 1 for TCE        
%% Section 1 setup
function [LCL,UCL] = compute_CI_glr(X,est_tce,h,alpha)
k = length(X) ;
glr = load('genLRtab') ; 
lgh_grid = (-5:0.5:3) ;
temp = lgh_grid(log(h)-lgh_grid>0) ;
h_low = temp(end) ;
h_high = lgh_grid(find(lgh_grid==h_low)+1) ;
h_low = exp(h_low) ; h_high = exp(h_high) ;

k_grid = [5,10,15,20,30,40,50,75,100] ;
row1 = est_tce*584+ (find(lgh_grid==log(h_low))-1)*36 -(log(h_low)==2)*4-(log(h_low)==2.5)*8-(log(h_low)==3)*16 ...
      + (find(k_grid==k)-1)*4 ...
      +(alpha==0.01) + (alpha==0.05)*2+(alpha==0.1)*3 + (alpha==0.2)*4 ; 
row2 = est_tce*584+ (find(lgh_grid==log(h_high))-1)*36 -(log(h_high)==2)*4-(log(h_high)==2.5)*8-(log(h_high)==3)*16 ...
      + (find(k_grid==k)-1)*4 ...
      +(alpha==0.01) + (alpha==0.05)*2+(alpha==0.1)*3 + (alpha==0.2)*4 ; 
cv1 = glr.mat(row1,5) ; cv2 = glr.mat(row2,5) ;
cv = (h_high-h)/(h_high-h_low)*cv1 + (h-h_low)/(h_high-h_low)*cv2 ;

%% Section 2 compute test stats
   nx = 200 ; nsig = 200 ; nm = 200 ;
   z = X(1:k) ;
   xsi_grid = linspace(-0.5,0.5,nx)' ;
   sig_grid = (X(1)-X(k))/(X(round(k/2))-X(k))*linspace(0.0001,35,nsig)' ;
   mu_grid = X(k)+(X(1)-X(k))/(X(round(k/2))-X(k))*linspace(-20,20,nm)' ;
   Ltemp = nan(nx*nsig*nm,1) ;
% unconstrained likelihood
        for j = 1:nx
           Ltemp(1+nsig*nm*(j-1):nsig*nm*j) = (1+xsi_grid(j)*kron(1./sig_grid,z(k)-mu_grid)).^(-1/xsi_grid(j))...
               +(1+1/xsi_grid(j))*sum(log(1+xsi_grid(j)*kron(1./sig_grid, repmat(z,nm,1)-repmat(mu_grid,1,k))),2)...
               +k*log(kron(sig_grid,ones(nm,1))) ;
        end
        Ltemp(imag(Ltemp)~=0) = 999 ;
        [~,sInd] = min(Ltemp) ;
        xsi_unc = xsi_grid(ceil(sInd/(nsig*nm))) ;
        sig_ind = ceil((sInd-(ceil(sInd/(nsig*nm))-1)*nsig*nm)/nm) ;
        sig_unc = sig_grid(sig_ind) ;
        mu_unc = mu_grid(sInd-(ceil(sInd/(nsig*nm))-1)*nsig*nm - (sig_ind-1)*nm) ;

    % unconstrained likelihood maximization                  
        p0 = [xsi_unc,sig_unc,mu_unc] ;
        lb = [-0.5,0,-Inf] ;
        ub = [0.5,Inf,Inf] ;
        
        options = optimoptions(@fmincon,'Algorithm','sqp') ;
        [p_unc,Lmin_unc] = fmincon(@(p) jevLR(p,z),p0,[],[],[],[],lb,ub,@(p)LRcon(p,z(1),z(k)),options) ;
      
    
    % constrained likelihood maximization
        [pc1,LCL] = fmincon(@(p) target(p,est_tce,h),p_unc,[],[],[],[],lb,ub,@(p)LRcon_cv(p,z,Lmin_unc,cv),options) ;
        c1 = LRcon_cv(pc1,z,Lmin_unc,cv) ;
        if c1(1)>0.01           
        problem_c1 = createOptimProblem('fmincon','objective',@(p)target(p,est_tce,h),'x0',p_unc...
            ,'lb',lb,'ub',ub,'nonlcon',@(p)LRcon_cv(p,z,Lmin_unc,cv),'options',options) ;        
        [pc1,LCL] = run(MultiStart('Display','off'),problem_c1,20) ; 
        end
        [pc2,UCL] = fmincon(@(p) -target(p,est_tce,h),p_unc,[],[],[],[],lb,ub,@(p)LRcon_cv(p,z,Lmin_unc,cv),options) ;  
        c2 = LRcon_cv(pc2,z,Lmin_unc,cv) ;
        if c2(1)>0.01           
        problem_c2 = createOptimProblem('fmincon','objective',@(p)-target(p,est_tce,h),'x0',p_unc(s,:)...
            ,'lb',lb,'ub',ub,'nonlcon',@(p)LRcon_cv(p,z(s,:),Ltemp,cv),'options',options) ;        
        [pc2,UCL] = run(MultiStart('Display','off'),problem_c2,20) ; 
        end
        UCL = -UCL ;    


end



